unit UDprBckp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fcopynew, Buttons, ExtCtrls, FB, FileCTRL, ComCtrls,
  folderBrowse, Menus, Extensions, IniFiles;


type

  TFrmDprBckp = class(TForm)
    Memo: TMemo;
    Panel1: TPanel;
    BtnClose: TBitBtn;
    Label2: TLabel;
    LBSources: TListBox;
    Label3: TLabel;
    BtnAddFolder: TBitBtn;
    BtnTrashFolder: TBitBtn;
    BtnChangeTargetFolder: TBitBtn;
    BtnCopy: TBitBtn;
    EdTarget: TEdit;
    BtnChProject: TBitBtn;
    OpenDialog: TOpenDialog;
    Bevel1: TBevel;
    Bevel2: TBevel;
    FCopy: TFileCopyNew;
    Panel2: TPanel;
    FBTarget: TBrowseForFolder;
    FBSources: TBrowseForFolder;
    BtnClearAllFolders: TBitBtn;
    Timer: TTimer;
    PopExt: TPopupMenu;
    Setincludedexstensions1: TMenuItem;
    SetDefault1: TMenuItem;
    Label1: TLabel;
    LBExtensions: TListBox;
    procedure FormActivate(Sender: TObject);
    procedure BtnTrashFolderClick(Sender: TObject);
    procedure BtnAddFolderClick(Sender: TObject);
    procedure BtnChangeTargetFolderClick(Sender: TObject);
    procedure BtnCopyClick(Sender: TObject);
    procedure BtnChProjectClick(Sender: TObject);
    procedure BtnClearAllFoldersClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Setincludedexstensions1Click(Sender: TObject);
    procedure SetDefault1Click(Sender: TObject);
    procedure FCopyError(Sender: TObject; ErrorMsg: String);
    procedure FCopyOverriding(Sender: TObject; var CanOverride: Boolean);
  private
    { Private declarations }
    curDir       : string;
    curProject   : string;  // current project to be copied
    targetFolder : string;  // the folder to copt to
    procedure parseProject; // extract subfolders used by the project
    function  SetCurProject : boolean;
    procedure ReadIniFile;
  public
    { Public declarations }
    ExtensionsList : TStringList;
  end;


var
  FrmDprBckp: TFrmDprBckp;

implementation

{$R *.DFM}


procedure TFrmDprBckp.parseProject;
var dpr : textFile;
    s, PathToAdd : string;

    function parseLine : string;
    var
       i : integer;
       sTmp : string;
       start : boolean;
    begin
       start := false;
       for i := 1 to length(s) do
       begin
          if s[i] = '''' then
          begin
             if start then break
             else start := true;
          end
          else
          begin
             if start then sTmp := sTmp + s[i];
          end;
       end;
       result := curDir + '\' + extractFileDir(sTmp);
    end;
begin
   memo.clear;
   AssignFile(dpr, curProject);
   Reset(dpr);
   while not eof(dpr) do
   begin
      Readln(dpr, S);
      if pos('\',s) > 0 then
      begin
         pathToAdd := parseLine;
         if LBSources.items.indexOf(pathToAdd) = -1 then
            LBSources.items.add(pathToAdd);
      end;
   end;
   CloseFile(dpr);
end;

function TFrmDprBckp.SetCurProject : boolean;
var
   SearchRec : TSearchRec;
   Found     : integer;
begin
   result := false;
   if ((curProject = '') or (not fileExists(curProject))) then
   begin
      Found := FindFirst(curDir + '\*.dpr', faAnyFile, SearchRec);
      FindClose(SearchRec);
      if Found = 0 then
      begin
         curProject := curDir + '\' + SearchRec.Name;
         TargetFolder  := curDir + '\001';
         result := true;
      end
      else
      begin
         showMessage('No project founded in ' + curDir + '.' + #13 +
                     'You must first choose a delphi''s project');
         BtnCopy.enabled := false;
         BtnChangeTargetFolder.enabled := false;
         BtnAddFolder.enabled := false;
         BtnChProject.setFocus;
         exit;
      end;
   end
   else
   begin
      result := true;
      curDir := extractFileDir(curProject);
   end;
   EdTarget.text := targetFolder;

   with LBSources.items do
   begin
      clear;
      add(curDir);
   end;
   parseProject;
   BtnCopy.enabled := true;
   BtnChangeTargetFolder.enabled := true;
   BtnAddFolder.enabled := true;
end;

procedure TFrmDprBckp.ReadIniFile;
var
   IniFile : TIniFile;
   tmp     : TStringList;
   s       : string;
   i       : integer;
begin
   tmp := TStringList.create;
   s := ExtractFilePath(paramStr(0));
   s := s + 'DprBckp.ini';
   if fileExists(s) then
   begin
      Inifile := TIniFile.create(s);
      curProject := IniFile.ReadString('curProject','curProject','');
      targetFolder := IniFile.ReadString('targetFolder','targetFolder','');
      IniFile.ReadSectionValues('Extensions', tmp);
      ExtensionsList.clear;
      for i := 0 to tmp.count-1 do
      begin
         ExtensionsList.add(tmp.Values['Ext' + IntToStr(i+1)]);
      end;
      Inifile.free;
   end
   else
   begin
      // initialize the list with default values
      ExtensionsList.add('*.pas');
      ExtensionsList.add('*.dfm');
      ExtensionsList.add('*.dpr');
      ExtensionsList.add('*.res');
      ExtensionsList.add('*.ini');
      ExtensionsList.add('*.sql');
      ExtensionsList.add('*.txt');
      // create the ini file
      Tmp.add('[Extensions]');
      for i := 0 to (ExtensionsList.count-1) do
      begin
          Tmp.add('ext'+ IntToStr(i + 1) + '=' + ExtensionsList[i]);
      end;
      Tmp.saveToFile(s);
   end;
   Tmp.free;
   LBExtensions.Items.clear;
   LBExtensions.Items.addStrings(ExtensionsList);
end;

procedure TFrmDprBckp.FormActivate(Sender: TObject);
begin
   getDir(0, curDir);
   if curDir[length(curDir)] = '\' then
      curDir := copy(curDir,1, length(curDir) - 1);
   ReadIniFile;
   if SetCurProject then
   begin
      EdTarget.text := targetFolder;
      timer.enabled := true;
   end;
end;

procedure TFrmDprBckp.BtnChProjectClick(Sender: TObject);
begin
   with OpenDialog do
   begin
      if execute then
      begin
         curProject := fileName;
         curDir     := extractFileDir(fileName);
         if SetCurProject then
         begin
            TargetFolder  := curDir + '\001';
            EdTarget.text := targetFolder;
         end;
      end;
   end;
end;


procedure TFrmDprBckp.BtnTrashFolderClick(Sender: TObject);
begin
   with LBSources do
   begin
      if itemIndex > -1 then
      begin
         items.delete(itemIndex);
      end;
   end;
   LBSources.setFocus;
end;

procedure TFrmDprBckp.BtnAddFolderClick(Sender: TObject);
begin
  if FBSources.execute then
     LBSources.items.add(FBSources.FolderName);
end;

procedure TFrmDprBckp.BtnChangeTargetFolderClick(Sender: TObject);
begin
   if FBTarget.execute then
      EdTarget.text := FBTarget.FolderName;
end;



procedure TFrmDprBckp.BtnCopyClick(Sender: TObject);
var
   i, y : integer;
   FFrom, FTo, s : string;
   list : TStringList;
begin
   for y := 0 to (ExtensionsList.count-1) do
   begin
      if (length(trim(ExtensionsList[y])) < 5) or (copy(ExtensionsList[y],1,1) <> '*') then
         continue;
      for i := 0 to (LBSources.items.count - 1) do
      begin
         FFrom := LBSources.items[i] + '\' + ExtensionsList[y];
         s := LBSources.items[i];
         delete(s,1,length(curDir));
         FTo   := EdTarget.text + s + '\';
         with FCopy do
         begin
            sourceFile := FFrom;
            targetFile := FTo;
            execCopy;
         end;
      end;
   end;
   showMessage('backup terminated');
   list := TStringList.create;
   list.add('[curProject]');
   list.add('curProject=' + curProject);
   list.add('');
   list.add('[targetFolder]');
   list.add('targetFolder=' + EdTarget.text);
   list.add('');
   list.add('[Extensions]');
   for i := 0 to ExtensionsList.count-1 do
   begin
       list.add('ext'+ IntToStr(i + 1) + '=' + ExtensionsList[i]);
   end;
   list.saveToFile(extractFilePath(paramStr(0)) + 'DprBckp.ini');
   list.free;
end;


procedure TFrmDprBckp.BtnClearAllFoldersClick(Sender: TObject);
begin
   LBSources.items.clear;
end;

procedure TFrmDprBckp.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   timer.enabled := false;
   ExtensionsList.clear;
   ExtensionsList.free;
end;

procedure TFrmDprBckp.TimerTimer(Sender: TObject);
begin
   BtnTrashFolder.enabled := (LBSources.ItemIndex > -1);
   BtnClearAllFolders.enabled := (LBSources.Items.Count > 0);
   BtnCopy.enabled := (LBSources.Items.Count > 0);
end;

procedure TFrmDprBckp.FormCreate(Sender: TObject);
begin
   ExtensionsList := TStringList.create;
end;

procedure TFrmDprBckp.SetIncludedexstensions1Click(Sender: TObject);
begin
   FrmExtensions := TFrmExtensions.create(self);
   FrmExtensions.Memo.Clear;
   FrmExtensions.memo.lines.assign(ExtensionsList);
   if FrmExtensions.ShowModal = mrOK then
   begin
      ExtensionsList.clear;
      ExtensionsList.assign(FrmExtensions.memo.lines);
   end;
   LBExtensions.Items.clear;
   LBExtensions.Items.addStrings(ExtensionsList);
end;

procedure TFrmDprBckp.SetDefault1Click(Sender: TObject);
begin
   ReadIniFile;
end;

procedure TFrmDprBckp.FCopyError(Sender: TObject; ErrorMsg: String);
var oldCursor : TCursor;
begin
   oldCursor     := screen.cursor;
   screen.cursor := crHourGlass;
   showMessage(ErrorMsg);
   screen.cursor := oldCursor;
end;

procedure TFrmDprBckp.FCopyOverriding(Sender: TObject;
  var CanOverride: Boolean);
begin
   CanOverride := true;
end;

end.
